
/* * * * * * * * * * * * * Author's note * * * * * * * * * * * *\
*   _       _   _       _   _       _   _       _     _ _ _ _   *
*  |_|     |_| |_|     |_| |_|_   _|_| |_|     |_|  _|_|_|_|_|  *
*  |_|_ _ _|_| |_|     |_| |_|_|_|_|_| |_|     |_| |_|_ _ _     *
*  |_|_|_|_|_| |_|     |_| |_| |_| |_| |_|     |_|   |_|_|_|_   *
*  |_|     |_| |_|_ _ _|_| |_|     |_| |_|_ _ _|_|  _ _ _ _|_|  *
*  |_|     |_|   |_|_|_|   |_|     |_|   |_|_|_|   |_|_|_|_|    *
*                                                               *
*                     http://www.humus.name                     *
*                                                                *
* This file is a part of the work done by Humus. You are free to   *
* use the code in any way you like, modified, unmodified or copied   *
* into your own work. However, I expect you to respect these points:  *
*  - If you use this file and its contents unmodified, or use a major *
*    part of this file, please credit the author and leave this note. *
*  - For use in anything commercial, please request my approval.     *
*  - Share your work and ideas too as much as you can.             *
*                                                                *
\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#include "App.h"

BaseApp *app = new App();

#ifdef WIN32

static char pth[MAX_PATH];

int CALLBACK BrowseCallbackProc(HWND hwnd, UINT uMsg, LPARAM lp, LPARAM pData){
	switch (uMsg){
	case BFFM_INITIALIZED:
		SendMessage(hwnd, BFFM_SETSELECTION, TRUE, (LPARAM) pth);
		break;
/*	case BFFM_SELCHANGED:
		char str[MAX_PATH];
		if (SHGetPathFromIDList((LPITEMIDLIST) lp, str)){
			SendMessage(hwnd, BFFM_SETSTATUSTEXT, 0, (LPARAM) str);
		}
		break;*/
	}

	return 0;
}
#endif

bool App::init(){
	depthBits = 0;
	zoom = 1.5f;

#if defined(WIN32)
	BROWSEINFO brInfo;
	brInfo.hwndOwner = NULL/*hwnd*/;
	brInfo.pidlRoot = NULL;
	brInfo.pszDisplayName = NULL;
	brInfo.lpszTitle = "Locate the CubeMap files";
	brInfo.ulFlags = BIF_EDITBOX;
	brInfo.lpfn = BrowseCallbackProc;
	brInfo.lParam = 0;
	brInfo.iImage = 0;

	memset(pth, 0, sizeof(pth));
	FILE *file = fopen("last.dir", "rb");
	if (file){
		fread(pth, sizeof(pth), 1, file);
		fclose(file);
	}

	LPITEMIDLIST pidl = SHBrowseForFolder(&brInfo);
	if (!pidl) return false;

	SHGetPathFromIDList(pidl, pth);
	IMalloc *imalloc = NULL;
	SHGetMalloc(&imalloc);
	imalloc->Free(pidl);
	imalloc->Release();

	file = fopen("last.dir", "wb");
	if (file){
		fwrite(pth, sizeof(pth), 1, file);
		fclose(file);
	}

	path = pth;
	path += "\\";

	WIN32_FIND_DATA findFileData;
	HANDLE handle;
	handle = FindFirstFile(path + "*.*", &findFileData);

	if (handle != INVALID_HANDLE_VALUE){
		do {
			processFileName(findFileData.cFileName);

		} while (FindNextFile(handle, &findFileData));
		FindClose(handle);
	}

#elif defined(LINUX)

	path = m_lpCmdLine;
	if (path[path.getLength() - 1] != '/') path += "/";

	DIR *dir;
	dirent *ent;

	if ((dir = opendir(path)) == NULL){
		printf("Can't access " + path + "\n");
		return false;
	}

	while ((ent = readdir(dir)) != NULL){
		processFileName(ent->d_name);
	}

	closedir(dir);

#endif


	return true;
}

void App::exit(){
}

void App::processFileName(const String &fileName){
	unsigned int index;
	if (fileName.getLength() > 2 && fileName.rfind('.', -1, &index)){
		String ext = ((const char *) fileName) + index + 1;
		ext.makeLowerCase();

		if (ext == "dds" || ext == "hdr" || ext == "htex" || ext == "png" || ext == "jpg" || ext == "jpeg" || ext == "tga" || ext == "pcx" || ext == "bmp"){
			if (fileName.find("negx") || fileName.find("xneg") || fileName.find("lf") || fileName.find("left" )) negx = fileName;
			if (fileName.find("posx") || fileName.find("xpos") || fileName.find("rt") || fileName.find("right")) posx = fileName;
			if (fileName.find("negy") || fileName.find("yneg") || fileName.find("dn") || fileName.find("down" ) || fileName.find("bottom")) negy = fileName;
			if (fileName.find("posy") || fileName.find("ypos") || fileName.find("up") || fileName.find("top"  )) posy = fileName;
			if (fileName.find("negz") || fileName.find("zneg") || fileName.find("bk") || fileName.find("back" )) negz = fileName;
			if (fileName.find("posz") || fileName.find("zpos") || fileName.find("ft") || fileName.find("front")) posz = fileName;
		}
	}
}

bool App::onMouseWheel(const int x, const int y, const int scroll){
	if (scroll > 0){
		if (zoom < 10) zoom *= 1.2f;
	} else {
		if (zoom > 0.03f) zoom /= 1.2f;
	}


	return true;
}

bool App::load(){
	if (!GL_ARB_texture_cube_map_supported){
		ErrorMsg("Cube mapping is not supported by your graphic card and driver.");
		return false;
	}

	if ((trilinear = renderer->addSamplerState(TRILINEAR, CLAMP, CLAMP, CLAMP)) == SS_NONE) return false;

	String nxPath = path + negx;
	String pxPath = path + posx;
	String nyPath = path + negy;
	String pyPath = path + posy;
	String nzPath = path + negz;
	String pzPath = path + posz;

	const char *fileNames[] = {
		pxPath, nxPath,
		pyPath, nyPath,
		pzPath, nzPath,
	};

	Image img;
	if (!img.loadSlicedImage(fileNames, 0)){
		char str[256];
		int n = sprintf(str, "Couldn't open cubemap:\n");
		for (int i = 0; i < 6; i++){
			n += sprintf(str + n, "%s\n", fileNames[i]);
		}

		ErrorMsg(str);
		return false;
	}
	if (img.getFormat() == FORMAT_RGBE8) img.convert(FORMAT_RGB32F);
	img.createMipMaps();


	// Make sure texture size is within GL limitations
	int maxSize = 1;
	glGetIntegerv(GL_MAX_CUBE_MAP_TEXTURE_SIZE, &maxSize);

	int w = img.getWidth();
	if (w > maxSize){

		char str[256];
		sprintf(str,
			"This cubemap have faces of %dx%d, largest supported on your system is %dx%d.\n"
			"The cubemap will be downsampled for display.", w, w, maxSize, maxSize);
		WarningMsg(str);

		int n = 0;
		do {
			w /= 2;
			n++;
		} while (w > maxSize);

		img.removeMipMaps(n);
	}

	if ((cubemap = renderer->addTexture(img, trilinear)) == TEXTURE_NONE) return false;

	return true;
}

void App::unload(){

}

void App::drawFrame(){
	float fov = 2 * atanf(zoom);

	mat4 projection = perspectiveMatrixX(fov, width, height, 1, 5000);
	mat4 modelview = rotateXY(-wx, -wy) * translate(-camPos);


	glMatrixMode(GL_PROJECTION);
	glLoadTransposeMatrixfARB(projection);

    glMatrixMode(GL_MODELVIEW);
	glLoadTransposeMatrixfARB(modelview);

	glClear(GL_COLOR_BUFFER_BIT);

	renderer->reset();
	((OpenGLRenderer *) renderer)->setTexture(cubemap);
	renderer->setDepthState(noDepthTest);
	renderer->apply();

	glBegin(GL_TRIANGLE_STRIP);
		glTexCoord3f(-1,1,-1);
		glVertex3f(-1024,  1024, -1024);
		glTexCoord3f(-1,-1,-1);
		glVertex3f(-1024, -1024, -1024);

		glTexCoord3f(-1,1,1);
		glVertex3f(-1024,  1024,  1024);
		glTexCoord3f(-1,-1,1);
		glVertex3f(-1024, -1024,  1024);

		glTexCoord3f(1,1,1);
		glVertex3f( 1024,  1024,  1024);
		glTexCoord3f(1,-1,1);
		glVertex3f( 1024, -1024,  1024);

		glTexCoord3f(1,1,-1);
		glVertex3f( 1024,  1024, -1024);
		glTexCoord3f(1,-1,-1);
		glVertex3f( 1024, -1024, -1024);
	glEnd();

	glBegin(GL_TRIANGLE_STRIP);
		glTexCoord3f(-1,1,1);
		glVertex3f(-1024,  1024,  1024);
		glTexCoord3f(1,1,1);
		glVertex3f( 1024,  1024,  1024);

		glTexCoord3f(-1,1,-1);
		glVertex3f(-1024,  1024, -1024);
		glTexCoord3f(1,1,-1);
		glVertex3f( 1024,  1024, -1024);

		glTexCoord3f(-1,-1,-1);
		glVertex3f(-1024, -1024, -1024);
		glTexCoord3f(1,-1,-1);
		glVertex3f( 1024, -1024, -1024);
		
		glTexCoord3f(-1,-1,1);
		glVertex3f(-1024, -1024,  1024);
		glTexCoord3f(1,-1,1);
		glVertex3f( 1024, -1024,  1024);
	glEnd();
}
